@@ -2,30 +2,42 @@
 """
 FindImports is a script that processes Python module dependencies.  Currently
 it can be used for finding unused imports and graphing module dependencies
-(with graphviz).  FindImports requires Python 2.6 or later.
+(with graphviz).
 
 Syntax: findimports.py [options] [filename|dirname ...]
 
 Options:
-  -h, --help        This help message
+  -h, --help            Show this help message and exit.
 
-  -i, --imports     Print dependency graph (default action).
-  -d, --dot         Print dependency graph in dot (graphviz) format.
-  -n, --names       Print dependency graph with all imported names.
+  -i, --imports         Print dependency graph (default action).
+  -d, --dot             Print dependency graph in dot (graphviz) format.
+  -n, --names           Print dependency graph with all imported names.
 
-  -u, --unused      Print unused imports.
-  -a, --all         Print unused imports even if there's a comment.
-  --duplicate       Print duplicate imports.
-  -v                Print more information (currently only affects --duplicate).
+  -u, --unused          Print unused imports.
+  -a, --all             Don't ignore unused imports when there's a comment on
+                        the same line (only affects -u).
+  --duplicate           Warn about duplicate imports.
+  -v, --verbose         Print more information (currently only 
+                        affects --duplicate).
 
-  -N, --noext       Omit external dependencies.
+  -N, --noext           Omit external dependencies.
 
-  -p, --packages    Convert the module graph to a package graph.
-  -l N, --level N   Collapse subpackages deeper than the Nth level.
+  -p, --packages        Convert the module graph to a package graph.
+  -l N, --level=N       Collapse subpackages to the topmost Nth levels.
+                 
+  -c, --collapse        Collapse dependency cycles.
+  -T, --tests           Collapse packages named 'tests' and 'ftests' with
+                        parent packages.
 
-  -c, --collapse    Collapse dependency cycles.
-  -T, --tests       Collapse packages named 'tests' and 'ftests' with parent
-                    packages.
+  -w FILE, --write-cache=FILE
+                        Write a pickle cache of parsed imports; provide the
+                        cache filename as the only non-option argument to load
+                        it back.
+  -I FILE, --ignore=FILE
+                        Ignore a file or directory; this option can be used
+                        multiple times. Default: ['venv']
+
+FindImports requires Python 3.6 or later.
 
 Notes:
 
@@ -48,7 +60,7 @@ Caching:
         findimports.py foo.importcache -d -N -c -p -l 2 > graph2.dot
 
 
-Copyright (c) 2003--2014 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2003--2019 Marius Gedminas <marius@pov.lt>
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -64,21 +76,24 @@ this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 """
 
-import os
-import sys
-import getopt
+from __future__ import print_function
+
+import ast
 import doctest
-import compiler
 import linecache
+import optparse
+import os
 import pickle
+import re
+import sys
+import tokenize
 import zipfile
 from operator import attrgetter
-from compiler.visitor import ASTVisitor
 
 
-__version__ = '1.3.1'
+__version__ = '2.1.0'
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
-__licence__ = 'GPL v2 or later'
+__licence__ = 'GPL v2 or v3'  # or ask me for MIT
 __url__ = 'https://github.com/mgedmin/findimports'
 
 
@@ -89,8 +104,9 @@ def adjust_lineno(filename, lineno, name):
     is always the first line number.
     """
     line = linecache.getline(filename, lineno)
-    # Hack warning: might be fooled by substrings or comments
-    while name not in line and line:
+    # Hack warning: might be fooled by comments
+    rx = re.compile(r'\b%s\b' % re.escape(name) if name != '*' else '[*]')
+    while line and not rx.search(line):
         lineno += 1
         line = linecache.getline(filename, lineno)
     return lineno
@@ -106,11 +122,13 @@ class ImportInfo(object):
         self.level = level
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r)' % (self.__class__.__name__, self.name,
-                                   self.filename, self.lineno, self.level)
+        return '%s(%r, %r, %r, %r)' % (
+            self.__class__.__name__, self.name, self.filename, self.lineno,
+            self.level,
+        )
 
 
-class ImportFinder(ASTVisitor):
+class ImportFinder(ast.NodeVisitor):
     """AST visitor that collects all imported names in its imports attribute.
 
     For example, the following import statements in the AST tree
@@ -144,53 +162,56 @@ class ImportFinder(ASTVisitor):
         info = ImportInfo(full_name, self.filename, lineno, level)
         self.imports.append(info)
 
-    def visitImport(self, node):
-        for name, imported_as in node.names:
-            self.processImport(name, imported_as, name, None, node)
+    def visit_Import(self, node):
+        for alias in node.names:
+            self.processImport(alias.name, alias.asname, alias.name, None,
+                               node)
 
-    def visitFrom(self, node):
-        if node.modname == '__future__':
+    def visit_ImportFrom(self, node):
+        if node.module == '__future__':
             return
 
-        for name, imported_as in node.names:
-            self.processImport(name, imported_as,
-                           '%s.%s' % (node.modname, name)
-                           if node.modname else name, node.level, node)
+        for alias in node.names:
+            name = alias.name
+            imported_as = alias.asname
+            fullname = '%s.%s' % (node.module, name) if node.module else name
+            self.processImport(name, imported_as, fullname, node.level, node)
 
     def visitSomethingWithADocstring(self, node):
-        self.processDocstring(node.doc, node.lineno)
-        for c in node.getChildNodes():
-            self.visit(c)
+        # ClassDef and FunctionDef have a 'lineno' attribute, Module doesn't.
+        lineno = getattr(node, 'lineno', None)
+        self.processDocstring(ast.get_docstring(node, clean=False), lineno)
+        self.generic_visit(node)
 
-    visitModule = visitSomethingWithADocstring
-    visitClass = visitSomethingWithADocstring
-    visitFunction = visitSomethingWithADocstring
+    visit_Module = visitSomethingWithADocstring
+    visit_ClassDef = visitSomethingWithADocstring
+    visit_FunctionDef = visitSomethingWithADocstring
 
     def processDocstring(self, docstring, lineno):
         if not docstring:
             return
         if lineno is None:
-            # Module nodes have a lineno of None.
+            # Module nodes don't have a lineno
             lineno = 0
         dtparser = doctest.DocTestParser()
         try:
             examples = dtparser.get_examples(docstring)
         except Exception:
-            print >> sys.stderr, ("%s:%s: error while parsing doctest"
-                                  % (self.filename, lineno))
+            print("{filename}:{lineno}: error while parsing doctest".format(
+                filename=self.filename, lineno=lineno), file=sys.stderr)
             raise
         for example in examples:
             try:
                 source = example.source
-                if isinstance(source, unicode):
-                    source = source.encode('UTF-8')
-                ast = compiler.parse(source)
+                if not isinstance(source, str):
+                    source = source.encode('UTF-8')  # pragma: PY2
+                node = ast.parse(source, filename='<docstring>')
             except SyntaxError:
-                print >> sys.stderr, ("%s:%s: syntax error in doctest"
-                                      % (self.filename, lineno))
+                print("{filename}:{lineno}: syntax error in doctest".format(
+                    filename=self.filename, lineno=lineno), file=sys.stderr)
             else:
                 self.lineno_offset += lineno + example.lineno
-                compiler.walk(ast, self)
+                self.visit(node)
                 self.lineno_offset -= lineno + example.lineno
 
 
@@ -216,10 +237,8 @@ class Scope(object):
         return self.parent.whereImported(name)
 
     def addImport(self, name, filename, level, lineno):
-        self.unused_names[name] = self.imports[name] = ImportInfo(name,
-                                                                  filename,
-                                                                  lineno,
-                                                                  level)
+        self.unused_names[name] = self.imports[name] = ImportInfo(
+            name, filename, lineno, level)
 
     def useName(self, name):
         if name in self.unused_names:
@@ -249,8 +268,9 @@ class ImportFinderAndNameTracker(ImportFinder):
         self.scope = self.scope_stack.pop()
 
     def leaveAllScopes(self):
-        while self.scope_stack:
-            self.leaveScope()
+        # newScope()/leaveScope() calls are always balanced so scope_stack
+        # should be empty at this point
+        assert not self.scope_stack
         self.unused_names += self.scope.unused_names.values()
         self.unused_names.sort(key=attrgetter('lineno'))
 
@@ -259,47 +279,53 @@ class ImportFinderAndNameTracker(ImportFinder):
         ImportFinder.processDocstring(self, docstring, lineno)
         self.leaveScope()
 
-    def visitFunction(self, node):
+    def visit_FunctionDef(self, node):
         self.newScope(self.scope, 'function %s' % node.name)
-        ImportFinder.visitFunction(self, node)
+        ImportFinder.visit_FunctionDef(self, node)
         self.leaveScope()
 
     def processImport(self, name, imported_as, full_name, level, node):
-        ImportFinder.processImport(self, name, imported_as, full_name, level, node)
+        ImportFinder.processImport(
+            self, name, imported_as, full_name, level, node)
         if not imported_as:
             imported_as = name
         if imported_as != "*":
             lineno = self.lineno_offset + node.lineno
             if (self.warn_about_duplicates and
-                self.scope.haveImport(imported_as)):
+                    self.scope.haveImport(imported_as)):
                 where = self.scope.whereImported(imported_as).lineno
-                print >> sys.stderr, ("%s:%s: %s imported again"
-                                      % (self.filename, lineno, imported_as))
-                if self.verbose:
-                    print >> sys.stderr, ("%s:%s:   (location of previous import)"
-                                        % (self.filename, where))
+                line = linecache.getline(self.filename, lineno)
+                if '#' not in line:
+                    print("{filename}:{lineno}: {name} imported again".format(
+                        filename=self.filename, lineno=lineno,
+                        name=imported_as), file=sys.stderr)
+                    if self.verbose:
+                        print("{filename}:{lineno}:   (location of previous"
+                              " import)".format(filename=self.filename,
+                                                lineno=where), file=sys.stderr)
             else:
                 self.scope.addImport(imported_as, self.filename, level, lineno)
 
-    def visitName(self, node):
-        self.scope.useName(node.name)
+    def visit_Name(self, node):
+        self.scope.useName(node.id)
 
-    def visitGetattr(self, node):
-        full_name = [node.attrname]
-        parent = node.expr
-        while isinstance(parent, compiler.ast.Getattr):
-            full_name.append(parent.attrname)
-            parent = parent.expr
-        if isinstance(parent, compiler.ast.Name):
-            full_name.append(parent.name)
+    def visit_Attribute(self, node):
+        full_name = [node.attr]
+        parent = node.value
+        while isinstance(parent, ast.Attribute):
+            full_name.append(parent.attr)
+            parent = parent.value
+        if isinstance(parent, ast.Name):
+            full_name.append(parent.id)
             full_name.reverse()
             name = ""
             for part in full_name:
-                if name: name = '%s.%s' % (name, part)
-                else: name += part
+                if name:
+                    name = '%s.%s' % (name, part)
+                else:
+                    name += part
                 self.scope.useName(name)
-        for c in node.getChildNodes():
-            self.visit(c)
+        self.generic_visit(node)
 
 
 def find_imports(filename):
@@ -307,9 +333,10 @@ def find_imports(filename):
 
     Returns a list of ImportInfo objects.
     """
-    ast = compiler.parseFile(filename)
+    with tokenize.open(filename) as f:
+        root = ast.parse(f.read(), filename)
     visitor = ImportFinder(filename)
-    compiler.walk(ast, visitor)
+    visitor.visit(root)
     return visitor.imports
 
 
@@ -319,11 +346,12 @@ def find_imports_and_track_names(filename, warn_about_duplicates=False,
 
     Returns ``(imports, unused)``.  Both are lists of ImportInfo objects.
     """
-    ast = compiler.parseFile(filename)
+    with tokenize.open(filename) as f:
+        root = ast.parse(f.read(), filename)
     visitor = ImportFinderAndNameTracker(filename)
     visitor.warn_about_duplicates = warn_about_duplicates
     visitor.verbose = verbose
-    compiler.walk(ast, visitor)
+    visitor.visit(root)
     visitor.leaveAllScopes()
     return visitor.imports, visitor.unused_names
 
@@ -350,6 +378,9 @@ class Module(object):
         self.imported_names = ()
         self.unused_names = ()
 
+    def __repr__(self):
+        return '<%s: %s>' % (self.__class__.__name__, self.modname)
+
 
 class ModuleCycle(object):
     """Node in a condenced module dependency graph.
@@ -378,19 +409,39 @@ class ModuleGraph(object):
 
     def __init__(self):
         self.modules = {}
-        self.path = sys.path
+        self.path = list(sys.path)
         self._module_cache = {}
         self._warned_about = set()
+        self._stderr = sys.stderr
+        self._exts = ('.py', '.so', '.dll')
+        if hasattr(sys, '_multiarch'):  # pragma: nocover
+            # Ubuntu 14.04 LTS renames
+            # /usr/lib/python2.7/lib-dynload/datetime.so to
+            # /usr/lib/python2.7/lib-dynload/datetime.x86_64-linux-gnu.so
+            # (https://github.com/mgedmin/findimports/issues/3)
+            self._exts += ('.%s.so' % sys._multiarch, )
 
-    def parsePathname(self, pathname):
+    def warn(self, about, message, *args):
+        if about in self._warned_about:
+            return
+        if args:
+            message = message % args
+        print(message, file=self._stderr)
+        self._warned_about.add(about)
+
+    def parsePathname(self, pathname, ignores=[]):
         """Parse one or more source files.
 
         ``pathname`` may be a file name or a directory name.
+        ``ignores`` is a list of files or directories to ignore.
         """
         if os.path.isdir(pathname):
             for root, dirs, files in os.walk(pathname):
                 dirs.sort()
                 files.sort()
+
+                self.filterIgnores(dirs, files, ignores)
+
                 for fn in files:
                     # ignore emacsish junk
                     if fn.endswith('.py') and not fn.startswith('.#'):
@@ -399,18 +450,23 @@ class ModuleGraph(object):
             self.readCache(pathname)
         else:
             self.parseFile(pathname)
+    
+    def filterIgnores(self, dirs, files, ignores):
+        for ignore in ignores:
+            if ignore in dirs:
+                dirs.remove(ignore)
+            if ignore in files:
+                files.remove(ignore)
 
     def writeCache(self, filename):
         """Write the graph to a cache file."""
-        f = file(filename, 'wb')
-        pickle.dump(self.modules, f)
-        f.close()
+        with open(filename, 'wb') as f:
+            pickle.dump(self.modules, f)
 
     def readCache(self, filename):
         """Load the graph from a cache file."""
-        f = file(filename, 'rb')
-        self.modules = pickle.load(f)
-        f.close()
+        with open(filename, 'rb') as f:
+            self.modules = pickle.load(f)
 
     def parseFile(self, filename):
         """Parse a single file."""
@@ -418,10 +474,11 @@ class ModuleGraph(object):
         module = Module(modname, filename)
         self.modules[modname] = module
         if self.trackUnusedNames:
-            module.imported_names, module.unused_names = \
-                    find_imports_and_track_names(filename,
-                                                 self.warn_about_duplicates,
-                                                 self.verbose)
+            module.imported_names, module.unused_names = (
+                find_imports_and_track_names(filename,
+                                             self.warn_about_duplicates,
+                                             self.verbose)
+            )
         else:
             module.imported_names = find_imports(filename)
             module.unused_names = None
@@ -432,19 +489,20 @@ class ModuleGraph(object):
 
     def filenameToModname(self, filename):
         """Convert a filename to a module name."""
-        for ext in ('.py', '.so', '.dll'):
+        for ext in reversed(self._exts):
             if filename.endswith(ext):
                 filename = filename[:-len(ext)]
                 break
         else:
-            print >> sys.stderr, "%s: unknown file name extension" % filename
+            self.warn(filename, '%s: unknown file name extension', filename)
         filename = os.path.abspath(filename)
         elements = filename.split(os.path.sep)
         modname = []
         while elements:
-            modname.append(elements[-1])
-            del elements[-1]
-            if not os.path.exists(os.path.sep.join(elements + ['__init__.py'])):
+            modname.append(elements.pop())
+            if not os.path.exists(
+                os.path.sep.join(elements + ['__init__.py'])
+            ):
                 break
         modname.reverse()
         modname = ".".join(modname)
@@ -458,12 +516,18 @@ class ModuleGraph(object):
 
         # extrapath is None only in a couple of test cases; in real life it's
         # always present
-        if level > 1 and extrapath:
+        if level and level > 1 and extrapath:
             # strip trailing path bits for each extra level to account for
             # relative imports
-            # from . import X has level == 1 and nothing is stripped (the level > 1 check accounts for this case)
-            # from .. import X has level == 2 and one trailing path component must go
-            # from ... import X has level == 3 and two trailing path components must go
+            # level 1: from . import X
+            # - nothing is stripped
+            #   (the level > 1 check accounts for this case)
+            # level 2: from .. import X
+            # - one trailing path component must go
+            # level 3: from ... import X
+            # - two trailing path components must go
+            # levels 4 through infinity:
+            # - you get the pattern
             extrapath = extrapath.split(os.path.sep)
             level -= 1
             extrapath = extrapath[0:-level]
@@ -476,11 +540,8 @@ class ModuleGraph(object):
             candidate = self.isPackage(name, extrapath)
             if candidate:
                 return candidate
-            name = name[:name.rfind('.')]
-        if dotted_name not in self._warned_about:
-            print >> sys.stderr, ("%s: could not find %s"
-                                  % (filename, dotted_name))
-            self._warned_about.add(dotted_name)
+            name = name.rpartition('.')[0]
+        self.warn(dotted_name, '%s: could not find %s', filename, dotted_name)
         return dotted_name
 
     def isModule(self, dotted_name, extrapath=None):
@@ -493,7 +554,7 @@ class ModuleGraph(object):
             return dotted_name
         filename = dotted_name.replace('.', os.path.sep)
         if extrapath:
-            for ext in ('.py', '.so', '.dll'):
+            for ext in self._exts:
                 candidate = os.path.join(extrapath, filename) + ext
                 if os.path.exists(candidate):
                     modname = self.filenameToModname(candidate)
@@ -512,13 +573,10 @@ class ModuleGraph(object):
                 try:
                     zf = zipfile.ZipFile(dir)
                 except zipfile.BadZipfile:
-                    if dir not in self._warned_about:
-                        print >> sys.stderr, ("%s: not a directory or zip file"
-                                              % dir)
-                        self._warned_about.add(dir)
+                    self.warn(dir, "%s: not a directory or zip file", dir)
                     continue
                 names = zf.namelist()
-                for ext in ('.py', '.so', '.dll'):
+                for ext in self._exts:
                     candidate = filename + ext
                     if candidate in names:
                         modname = filename.replace(os.path.sep, '.')
@@ -526,7 +584,7 @@ class ModuleGraph(object):
                         self._module_cache[(dotted_name, None)] = modname
                         return modname
             else:
-                for ext in ('.py', '.so', '.dll'):
+                for ext in self._exts:
                     candidate = os.path.join(dir, filename) + ext
                     if os.path.exists(candidate):
                         modname = self.filenameToModname(candidate)
@@ -559,7 +617,7 @@ class ModuleGraph(object):
             if name in pkgnames:
                 break
             result.append(name)
-        if not result: # empty names are baad
+        if not result:  # empty names are baad
             return dotted_name
         return '.'.join(result)
 
@@ -580,14 +638,17 @@ class ModuleGraph(object):
             package = packages[package_name]
             for name in module.imports:
                 package_name = self.packageOf(name, packagelevel)
-                if package_name != package.modname: # no loops
+                if package_name != package.modname:  # no loops
                     package.imports.add(package_name)
         graph = ModuleGraph()
         graph.modules = packages
         return graph
 
     def collapseTests(self, pkgnames=['tests', 'ftests']):
-        """Collapse test packages with parent packages."""
+        """Collapse test packages with parent packages.
+
+        Works only with package graphs.
+        """
         packages = {}
         for module in self.listModules():
             package_name = self.removeTestPackage(module.modname, pkgnames)
@@ -598,7 +659,7 @@ class ModuleGraph(object):
             package = packages[package_name]
             for name in module.imports:
                 package_name = self.removeTestPackage(name, pkgnames)
-                if package_name != package.modname: # no loops
+                if package_name != package.modname:  # no loops
                     package.imports.add(package_name)
         graph = ModuleGraph()
         graph.modules = packages
@@ -611,28 +672,33 @@ class ModuleGraph(object):
         """
         # This algorithm determines Strongly Connected Components.  Look it up.
         # It is adapted to suit our data structures.
+
         # Phase 0: prepare the graph
         imports = {}
         for u in self.modules:
             imports[u] = set()
             for v in self.modules[u].imports:
-                if v in self.modules: # skip external dependencies
+                if v in self.modules:  # skip external dependencies
                     imports[u].add(v)
+
         # Phase 1: order the vertices
         visited = {}
         for u in self.modules:
             visited[u] = False
         order = []
+
         def visit1(u):
             visited[u] = True
             for v in imports[u]:
                 if not visited[v]:
                     visit1(v)
             order.append(u)
+
         for u in self.modules:
             if not visited[u]:
                 visit1(u)
         order.reverse()
+
         # Phase 2: compute the inverse graph
         revimports = {}
         for u in self.modules:
@@ -640,17 +706,20 @@ class ModuleGraph(object):
         for u in self.modules:
             for v in imports[u]:
                 revimports[v].add(u)
+
         # Phase 3: determine the strongly connected components
         components = {}
         component_of = {}
         for u in self.modules:
             visited[u] = False
+
         def visit2(u):
             visited[u] = True
             component.append(u)
             for v in revimports[u]:
                 if not visited[v]:
                     visit2(v)
+
         for u in order:
             if not visited[u]:
                 component = []
@@ -660,6 +729,7 @@ class ModuleGraph(object):
                 components[node.modname] = node
                 for modname in component:
                     component_of[modname] = node
+
         # Phase 4: construct the condensed graph
         for node in components.values():
             for modname in node.modnames:
@@ -674,20 +744,21 @@ class ModuleGraph(object):
     def printImportedNames(self):
         """Produce a report of imported names."""
         for module in self.listModules():
-            print "%s:" % module.modname
-            print "  %s" % "\n  ".join(imp.name for imp in module.imported_names)
+            print("%s:" % module.modname)
+            print("  %s" % "\n  ".join(
+                imp.name for imp in module.imported_names))
 
     def printImports(self):
         """Produce a report of dependencies."""
         for module in self.listModules():
-            print "%s:" % module.label
+            print("%s:" % module.label)
             if self.external_dependencies:
                 imports = list(module.imports)
             else:
                 imports = [modname for modname in module.imports
                            if modname in self.modules]
             imports.sort()
-            print "  %s" % "\n  ".join(imports)
+            print("  %s" % "\n  ".join(imports))
 
     def printUnusedImports(self):
         """Produce a report of unused imports."""
@@ -699,36 +770,36 @@ class ModuleGraph(object):
                 if not self.all_unused:
                     line = linecache.getline(module.filename, lineno)
                     if '#' in line:
-                        continue # assume there's a comment explaining why it
-                                 # is not used
-                print "%s:%s: %s not used" % (module.filename, lineno, name)
+                        # assume there's a comment explaining why it's not used
+                        continue
+                print("%s:%s: %s not used" % (module.filename, lineno, name))
 
     def printDot(self):
         """Produce a dependency graph in dot format."""
-        print "digraph ModuleDependencies {"
-        print "  node[shape=box];"
+        print("digraph ModuleDependencies {")
+        print("  node[shape=box];")
         allNames = set()
         nameDict = {}
         for n, module in enumerate(self.listModules()):
             module._dot_name = "mod%d" % n
             nameDict[module.modname] = module._dot_name
-            print "  %s[label=\"%s\"];" % (module._dot_name,
-                                           quote(module.label))
+            print("  %s[label=\"%s\"];" % (module._dot_name,
+                                           quote(module.label)))
             allNames |= module.imports
-        print "  node[style=dotted];"
+        print("  node[style=dotted];")
         if self.external_dependencies:
             myNames = set(self.modules)
             extNames = list(allNames - myNames)
             extNames.sort()
             for n, name in enumerate(extNames):
                 nameDict[name] = id = "extmod%d" % n
-                print "  %s[label=\"%s\"];" % (id, name)
+                print("  %s[label=\"%s\"];" % (id, name))
         for modname, module in sorted(self.modules.items()):
             for other in sorted(module.imports):
                 if other in nameDict:
-                    print "  %s -> %s;" % (nameDict[module.modname],
-                                           nameDict[other])
-        print "}"
+                    print("  %s -> %s;" % (nameDict[module.modname],
+                                           nameDict[other]))
+        print("}")
 
 
 def quote(s):
@@ -739,74 +810,83 @@ def quote(s):
     return s.replace("\\", "\\\\").replace('"', '\\"').replace('\n', '\\n')
 
 
-def main(argv=sys.argv):
-    progname = os.path.basename(argv[0])
-    helptext = __doc__.strip().replace('findimports.py', progname)
-    g = ModuleGraph()
-    action = 'printImports'
-    condense_to_packages = False
-    collapse_cycles = False
-    collapse_tests = False
-    packagelevel = None
-    noext = False
-    write_cache = None
+def main(argv=None):
+    progname = os.path.basename(argv[0]) if argv else None
+    description = __doc__.strip().split('\n\n')[0]
+    parser = optparse.OptionParser('%prog [options] [filename|dirname ...]',
+                                   prog=progname, description=description)
+    parser.add_option('-i', '--imports', action='store_const',
+                      dest='action', const='printImports',
+                      default='printImports',
+                      help='print dependency graph (default action)')
+    parser.add_option('-d', '--dot', action='store_const',
+                      dest='action', const='printDot',
+                      help='print dependency graph in dot (graphviz) format')
+    parser.add_option('-n', '--names', action='store_const',
+                      dest='action', const='printImportedNames',
+                      help='print dependency graph with all imported names')
+    parser.add_option('-u', '--unused', action='store_const',
+                      dest='action', const='printUnusedImports',
+                      help='print unused imports')
+    parser.add_option('-a', '--all', action='store_true',
+                      dest='all_unused',
+                      help="don't ignore unused imports when there's a comment"
+                           " on the same line (only affects -u)")
+    parser.add_option('--duplicate', action='store_true',
+                      dest='warn_about_duplicates',
+                      help='warn about duplicate imports')
+    parser.add_option('-v', '--verbose', action='store_true',
+                      help='print more information (currently only affects'
+                           ' --duplicate)')
+    parser.add_option('-N', '--noext', action='store_true',
+                      help='omit external dependencies')
+    parser.add_option('-p', '--packages', action='store_true',
+                      dest='condense_to_packages',
+                      help='convert the module graph to a package graph')
+    parser.add_option('-l', '--level', type='int',
+                      dest='packagelevel',
+                      help='collapse subpackages to the topmost Nth levels')
+    parser.add_option('-c', '--collapse', action='store_true',
+                      dest='collapse_cycles',
+                      help='collapse dependency cycles')
+    parser.add_option('-T', '--tests', action='store_true',
+                      dest='collapse_tests',
+                      help="collapse packages named 'tests' and 'ftests'"
+                           " with parent packages")
+    parser.add_option('-w', '--write-cache', metavar='FILE',
+                      help="write a pickle cache of parsed imports; provide"
+                           " the cache filename as the only non-option"
+                           " argument to load it back")
+    parser.add_option('-I', '--ignore', metavar='FILE', action="append",
+                      help="ignore a file or directory;"
+                           " this option can be used multiple times."
+                           " Default: ['venv']")
     try:
-        opts, args = getopt.gnu_getopt(argv[1:], 'duniahpl:cNTv',
-                                   ['dot', 'unused', 'all', 'names', 'imports',
-                                    'packages', 'level=', 'help', 'collapse',
-                                    'noext', 'tests', 'write-cache=',
-                                    'duplicate', 'verbose'])
-    except getopt.error, e:
-        print >> sys.stderr, "%s: %s" % (progname, e)
-        print >> sys.stderr, "Try %s --help." % progname
-        return 1
-    for k, v in opts:
-        if k in ('-d', '--dot'):
-            action = 'printDot'
-        elif k in ('-u', '--unused'):
-            action = 'printUnusedImports'
-        elif k in ('-a', '--all'):
-            g.all_unused = True
-        elif k == '--duplicate':
-            g.warn_about_duplicates = True
-        elif k in ('-v', '--verbose'):
-            g.verbose = True
-        elif k in ('-n', '--names'):
-            action = 'printImportedNames'
-        elif k in ('-i', '--imports'):
-            action = 'printImports'
-        elif k in ('-p', '--packages'):
-            condense_to_packages = True
-        elif k in ('-l', '--level'):
-            packagelevel = int(v)
-        elif k in ('-c', '--collapse'):
-            collapse_cycles = True
-        elif k in ('-N', '--noext'):
-            noext = True
-        elif k in ('-T', '--tests'):
-            collapse_tests = True
-        elif k == '--write-cache':
-            write_cache = v
-        elif k in ('-h', '--help'):
-            print helptext
-            return 0
-    g.trackUnusedNames = (action == 'printUnusedImports')
+        opts, args = parser.parse_args(args=argv[1:] if argv else None)
+    except SystemExit as e:
+        return e.code
     if not args:
         args = ['.']
+
+    g = ModuleGraph()
+    g.all_unused = opts.all_unused
+    g.warn_about_duplicates = opts.warn_about_duplicates
+    g.verbose = opts.verbose
+    g.trackUnusedNames = (opts.action == 'printUnusedImports')
     for fn in args:
-        g.parsePathname(fn)
-    if write_cache:
-        g.writeCache(write_cache)
-    if condense_to_packages:
-        g = g.packageGraph(packagelevel)
-    if collapse_tests:
+        g.parsePathname(fn, ignores=opts.ignore or ["venv"])
+    if opts.write_cache:
+        g.writeCache(opts.write_cache)
+    if opts.condense_to_packages:
+        g = g.packageGraph(opts.packagelevel)
+    if opts.collapse_tests:
         g = g.collapseTests()
-    if collapse_cycles:
+    if opts.collapse_cycles:
         g = g.collapseCycles()
-    g.external_dependencies = not noext
-    getattr(g, action)()
+    g.external_dependencies = not opts.noext
+    getattr(g, opts.action)()
     return 0
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: nocover
     sys.exit(main())
